#include <petscdmda.h>
#include <petscksp.h>
#include <petscpcmg.h>
#include <petscdmmg.h>
#include "Initialization.h"
#include "def.h"

Geometry Cylinder;

#undef __FUNCT__
#define __FUNCT__ "Initialization"
PetscErrorCode Initialization(DM da) {
  FILE *fpgeo, *fpuc, *fpvc, *fppc;
  DM             cda;
  PetscErrorCode ierr;
  PetscInt       i,j,mx,my,xm,ym,mm,nn,xs,ys, rank;
  PetscScalar    Hx,Hy;
  DMDACoor2d    **coors;
  Vec gc;
  Flag **localFlag;
  char fname[20];
  StaggerGrid ucgrid, vcgrid, pcgrid;
 
  PetscFunctionBegin;
// -- 0.0 -- Read Info. of DMDA
  ierr = DMDAGetInfo(da, 0, &mx, &my, 0, &mm , &nn, 0,0,0,0,0,0,0);CHKERRQ(ierr);
  Hx   = 1.0 / (PetscReal)(mx-1);
  Hy   = 1.0 / (PetscReal)(my-1);
  MPI_Comm_rank(PETSC_COMM_WORLD, &rank);
  ierr = DMDAGetCorners(da,&xs,&ys,0,&xm,&ym,0);CHKERRQ(ierr);

  printf("node = %d mx = %d  my = %d  mm = %d  nn = %d  \n", rank,  mx, my, mm, nn);
  printf("node = %d xs = %d  ys = %d  xm = %d  ym = %d  \n", rank, xs, ys, xm, ym);

// -- 0.0 -- Finished

// -- 1.0 -- Drawing the cylinder
  sprintf(fname, "%s%d%s", "Results/Geometry", rank, ".dat");
  fpgeo = fopen(fname, "w");
  for(i = 0; i < crclnm; i++) {
    Cylinder.x[i] = cntx + rr*cos(2.*PI/crclnm*i);
    Cylinder.y[i] = cnty + rr*sin(2.*PI/crclnm*i);
    fprintf(fpgeo, "%f  %f\n", Cylinder.x[i], Cylinder.y[i]);
  }
  fclose(fpgeo);

// -- 1.0 -- Finished 

// -- 2.0 -- Generating the grid
  DMDAGetCoordinateDA(da, &cda);
  DMDAGetGhostedCoordinates(da, &gc);
  DMDAVecGetArray(cda, gc, &coors);

  for (i = xs; i < xs+xm; i++) {
    for (j = ys; j < ys+ym; j++) {
//      printf("%f  %f  \n", coors[j][i].x, coors[j][i].y);
    }
  }


  // -- 2.1 -- Determinnig flags
    // -- 2.1.1 -- Allocate Memory 
  localFlag = (Flag**) calloc((xm), sizeof(Flag*));
  for(i = 0; i < (xm); i++) {
    localFlag[i] = (Flag*) calloc((ym), sizeof(Flag));
  }  
    // -- 2.1.1 -- Finished
  sprintf(fname, "%s%d%s", "Results/UCFlag", rank, ".dat");
  fpuc = fopen(fname, "w");
  sprintf(fname, "%s%d%s", "Results/VCFlag", rank, ".dat");
  fpvc = fopen(fname, "w");
  sprintf(fname, "%s%d%s", "Results/PCFlag", rank, ".dat");
  fppc = fopen(fname, "w");

  for (i = 0; i < (xm); i++) {
    for (j = 0; j < (ym); j++) {
      ucgrid.x = coors[ys+j][xs+i].x;
      ucgrid.y = coors[ys+j][xs+i].y-0.5*dyValue;
      if(sqrt(pow((ucgrid.x-cntx), 2.0)+pow((ucgrid.y-cnty), 2.0)) <= rr) localFlag[i][j].uc = 1;
      else localFlag[i][j].uc = 0;

      vcgrid.x = coors[ys+j][xs+i].x-0.5*dxValue;
      vcgrid.y = coors[ys+j][xs+i].y;
      if(sqrt(pow((vcgrid.x-cntx), 2.0)+pow((vcgrid.y-cnty), 2.0)) <= rr) localFlag[i][j].vc = 1;
      else localFlag[i][j].vc = 0;

      pcgrid.x = coors[ys+j][xs+i].x-0.5*dxValue;
      pcgrid.y = coors[ys+j][xs+i].y-0.5*dyValue;
      if(sqrt(pow((pcgrid.x-cntx), 2.0)+pow((pcgrid.y-cnty), 2.0)) <= rr) localFlag[i][j].pc = 1;
      else localFlag[i][j].pc = 0;

      if(localFlag[i][j].uc == 1) fprintf(fpuc, "%f  %f  \n", ucgrid.x, ucgrid.y);
      if(localFlag[i][j].vc == 1) fprintf(fpvc, "%f  %f  \n", vcgrid.x, vcgrid.y);
      if(localFlag[i][j].pc == 1) fprintf(fppc, "%f  %f  \n", pcgrid.x, pcgrid.y);
    }
  }

  fclose(fpuc);
  fclose(fpvc);
  fclose(fppc);

  // -- 2.1 -- Finished


  DMDAVecRestoreArray(cda, gc, &coors);
// -- 2.0 -- Finished
  
  PetscFunctionReturn(0);

}
